/**
	Copyright (C) 2013 Eric Wasylishen

	Date:  August 2013
	License:  MIT  (see COPYING)
 */

#import <Foundation/Foundation.h>

@class COUndoTrack;
@class FMDatabase;
@class ETUUID;

NSString * const COUndoTrackStoreTrackDidChangeNotification;

// User info keys for COUndoTrackStoreTrackDidChangeNotification
NSString * const COUndoTrackStoreTrackName;
/**
 * UUID string
 */
NSString * const COUndoTrackStoreTrackHeadCommandUUID;
/**
 * NSNull or UUID string
 */
NSString * const COUndoTrackStoreTrackCurrentCommandUUID;

@interface COUndoTrackSerializedCommand : NSObject
@property (readwrite, nonatomic) id JSONData;
@property (readwrite, nonatomic) NSDictionary *metadata;
@property (readwrite, nonatomic) ETUUID *UUID;
@property (readwrite, nonatomic) ETUUID *parentUUID;
@property (readwrite, nonatomic) NSString *trackName;
@property (readwrite, nonatomic) NSDate *timestamp;
@property (readwrite, nonatomic) int64_t sequenceNumber;
@end

@interface COUndoTrackState : NSObject <NSCopying>
@property (readwrite, nonatomic) NSString *trackName;
@property (readwrite, nonatomic) ETUUID *headCommandUUID;
@property (readwrite, nonatomic) ETUUID *currentCommandUUID;
@end

@interface COUndoTrackStore : NSObject
{
    FMDatabase *_db;
	NSMutableDictionary *_modifiedTrackStateForTrackName;
}

+ (COUndoTrackStore *) defaultStore;

/** @taskunit Framework Private */

- (BOOL) beginTransaction;
- (BOOL) commitTransaction;

- (NSArray *) trackNames;
- (NSArray *) trackNamesMatchingGlobPattern: (NSString *)aPattern;
- (COUndoTrackState *) stateForTrackName: (NSString*)aName;
- (void) setTrackState: (COUndoTrackState *)aState;
- (void) removeTrackWithName: (NSString*)aName;
- (NSArray *) allCommandUUIDsOnTrackWithName: (NSString*)aName;

/**
 * sequenceNumber is set in the provided command object
 */
- (void) addCommand: (COUndoTrackSerializedCommand *)aCommand;
- (COUndoTrackSerializedCommand *) commandForUUID: (ETUUID *)aUUID;
- (void) removeCommandForUUID: (ETUUID *)aUUID;

- (BOOL) string: (NSString *)aString matchesGlobPattern: (NSString *)aPattern;

@end
